"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.LodContentExtractorImpl = void 0;
const tslib_1 = require("tslib");
const inversify_1 = require("inversify");
const types_1 = require("@services/types");
const xml2js = (0, tslib_1.__importStar)(require("xml2js"));
const fs = (0, tslib_1.__importStar)(require("fs"));
const path = (0, tslib_1.__importStar)(require("path"));
const constants_1 = require("@model/constants");
const https = (0, tslib_1.__importStar)(require("https"));
const child_process_1 = require("child_process");
let LodContentExtractorImpl = class LodContentExtractorImpl {
    fileWriter;
    nounExtraxtor;
    verbExtractor;
    prepositionExtractor;
    baseLodWordExtractor;
    constructor(fileWriter, nounExtraxtor, verbExtractor, prepositionExtractor, baseLodWordExtractor) {
        this.fileWriter = fileWriter;
        this.nounExtraxtor = nounExtraxtor;
        this.verbExtractor = verbExtractor;
        this.prepositionExtractor = prepositionExtractor;
        this.baseLodWordExtractor = baseLodWordExtractor;
    }
    async extract(lodDumpFile, outputDirectory) {
        const lodAudiosFolder = path.join(outputDirectory, constants_1.CONSTANTS.AUDIOS_FOLDER);
        const extractedWords = new Map();
        const lodKeysToWords = new Map();
        const parser = new xml2js.Parser({ attrkey: "attributes", explicitChildren: true, preserveChildrenOrder: true });
        try {
            let xmlStr = fs.readFileSync(lodDumpFile);
            const xml = await parser.parseStringPromise(xmlStr);
            const items = xml["lod:LOD"]["lod:ITEM"];
            for (let itemIndex = 0; itemIndex < items.length; itemIndex++) {
                const article = items[itemIndex]["lod:ARTICLE"][0];
                const hasAudio = !!items[itemIndex]["lod:AUDIO"];
                const lodKey = items[itemIndex]["lod:META"][0]["attributes"]["lod:ID"].trim();
                if (hasAudio) {
                    if (!this.audioExists(lodKey, lodAudiosFolder)) {
                        await this.persistMp3(lodKey, lodAudiosFolder);
                        this.persistM4a(lodKey, lodAudiosFolder);
                    }
                }
                let wordObjs = this.extractArticle(lodKey, article);
                for (const wordObj of wordObjs) {
                    for (const type of wordObj.types) {
                        type.details.audio = `${lodKey.toLowerCase()}.m4a`;
                    }
                }
                for (const wordObj of wordObjs) {
                    lodKeysToWords.set(lodKey, wordObj.word);
                    if (extractedWords.has(wordObj.id)) {
                        const previousWordObj = extractedWords.get(wordObj.id);
                        for (const type of wordObj.types) {
                            previousWordObj.types.push(type);
                        }
                    }
                    else {
                        extractedWords.set(wordObj.id, wordObj);
                    }
                }
            }
            const wordsJsonFolder = path.join(outputDirectory, constants_1.CONSTANTS.WORDS_FOLDER);
            for (const word of extractedWords.values()) {
                for (const type of word.types) {
                    if (!!type.details.variationOfLodKey) {
                        type.details.variationOf = lodKeysToWords.get(type.details.variationOfLodKey);
                    }
                }
                this.persistJson(word.id, wordsJsonFolder, Buffer.from(JSON.stringify(word)).toString("base64"));
            }
            ;
        }
        catch (exception) {
            console.error(exception);
        }
    }
    extractArticle(lodKey, article) {
        const word = article["lod:ITEM-ADRESSE"][0]["_"].replace(/\_/g, " ").replace(/\#/g, "'");
        const structures = article["lod:MICROSTRUCTURE"];
        const words = [];
        for (const structure of structures) {
            const typeKey = Object.keys(structure)[1];
            for (const type of structure[typeKey]) {
                let extractor;
                let lodWordType = typeKey.slice(typeKey.lastIndexOf("-") + 1);
                switch (typeKey) {
                    case "lod:MS-TYPE-PREP-plus-ART":
                    case "lod:MS-TYPE-PART":
                    case "lod:MS-TYPE-VRBPART":
                    case "lod:MS-TYPE-ELEM-COMP":
                    case "lod:MS-TYPE-PRONADV":
                    case "lod:MS-TYPE-INDEF":
                    case "lod:MS-TYPE-INTERJ":
                    case "lod:MS-TYPE-PRON":
                    case "lod:MS-TYPE-ART":
                        continue;
                    case "lod:MS-TYPE-SUBST":
                        extractor = this.nounExtraxtor;
                        break;
                    case "lod:MS-TYPE-PREP":
                        extractor = this.prepositionExtractor;
                        break;
                    case "lod:MS-TYPE-VRB":
                        extractor = this.verbExtractor;
                        break;
                    case "lod:MS-TYPE-CONJ":
                    case "lod:MS-TYPE-ADJ":
                    case "lod:MS-TYPE-ADV":
                        extractor = this.baseLodWordExtractor;
                        break;
                    default: throw new Error(`${type} not recognized.`);
                }
                const wordObj = extractor.extract(lodKey, lodWordType, word, type);
                if (!!wordObj) {
                    words.push(wordObj);
                }
            }
        }
        return words;
    }
    audioExists(lodKey, lodAudiosFolder) {
        return fs.existsSync(path.join(lodAudiosFolder, `${lodKey.toLowerCase()}.m4a`));
    }
    async persistMp3(lodKey, outputDirectory) {
        const waitPromise = new Promise((resolve, reject) => {
            https.get(`https://www.lod.lu/audio/${lodKey.toLowerCase()}.mp3`, (response) => {
                response.setEncoding("base64");
                let body = "";
                response.on("error", reject);
                response.on("data", (data) => { body += data; });
                response.on("end", () => {
                    this.fileWriter.write(outputDirectory, `${lodKey.toLowerCase()}.mp3`, body);
                    resolve();
                });
            });
        });
        return waitPromise;
    }
    persistM4a(lodKey, outputDirectory) {
        const mp3File = path.join(outputDirectory, `${lodKey.toLowerCase()}.mp3`);
        const m4aFile = path.join(outputDirectory, `${lodKey.toLowerCase()}.m4a`);
        try {
            (0, child_process_1.execSync)(`ffmpeg -i "${mp3File}" -map a:0 -c:a aac "${m4aFile}"`);
        }
        catch (e) {
            console.log(`${lodKey} audio file is broken.`);
        }
        fs.rmSync(mp3File);
    }
    persistJson(wordKey, outputDirectory, body) {
        this.fileWriter.write(outputDirectory, `${wordKey}.json`, body);
    }
};
LodContentExtractorImpl = (0, tslib_1.__decorate)([
    (0, inversify_1.injectable)(),
    (0, tslib_1.__param)(0, (0, inversify_1.inject)(types_1.TYPES.FileWriter)),
    (0, tslib_1.__param)(1, (0, inversify_1.inject)(types_1.TYPES.NounExtractor)),
    (0, tslib_1.__param)(2, (0, inversify_1.inject)(types_1.TYPES.VerbExtractor)),
    (0, tslib_1.__param)(3, (0, inversify_1.inject)(types_1.TYPES.PrepositionExtractor)),
    (0, tslib_1.__param)(4, (0, inversify_1.inject)(types_1.TYPES.BaseLodWordExtractor)),
    (0, tslib_1.__metadata)("design:paramtypes", [Object, Object, Object, Object, Object])
], LodContentExtractorImpl);
exports.LodContentExtractorImpl = LodContentExtractorImpl;
